"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.interceptor = void 0;
var _xeUtils = _interopRequireDefault(require("xe-utils"));
var _log = require("../../tools/log");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
var storeMap = {};
var interceptor = {
  mixin: function (options) {
    _xeUtils.default.each(options, function (callback, type) {
      return interceptor.add(type, callback);
    });
    return interceptor;
  },
  get: function (type) {
    return storeMap[type] || [];
  },
  add: function (type, callback) {
    // 检测类型
    if (process.env.NODE_ENV === 'development') {
      var eventTypes = ['created', 'mounted', 'activated', 'beforeUnmount', 'unmounted', 'event.clearActived', 'event.clearFilter', 'event.clearAreas', 'event.showMenu', 'event.keydown', 'event.export', 'event.import'];
      if (eventTypes.indexOf(type) === -1) {
        (0, _log.warnLog)('vxe.error.errProp', ["Interceptor.".concat(type), eventTypes.join('|')]);
      }
    }
    if (callback) {
      var eList = storeMap[type];
      if (!eList) {
        eList = storeMap[type] = [];
      }
      // 检测重复
      if (process.env.NODE_ENV === 'development') {
        if (eList.indexOf(callback) > -1) {
          (0, _log.warnLog)('vxe.error.coverProp', ['Interceptor', type]);
        }
      }
      eList.push(callback);
    }
    return interceptor;
  },
  delete: function (type, callback) {
    var eList = storeMap[type];
    if (eList) {
      if (callback) {
        _xeUtils.default.remove(eList, function (fn) {
          return fn === callback;
        });
      } else {
        delete storeMap[type];
      }
    }
  }
};
exports.interceptor = interceptor;